@@ -30,6 +30,14 @@ class Parser(object):
         self._parse_file.restype = None
         self._parse_file.argtypes = [c.c_void_p, c.c_char_p, c.c_char_p, c.c_bool]
 
+        self._parse_tagged_sentence = libptr.parse_tagged_sentence
+        self._parse_tagged_sentence.restype = c.c_char_p
+        self._parse_tagged_sentence.argtypes = [c.c_void_p, c.c_char_p, c.c_char]
+
+        self._parse_tagged_file = libptr.parse_tagged_file
+        self._parse_tagged_file.restype = None
+        self._parse_tagged_file.argtypes = [c.c_void_p, c.c_char_p, c.c_char_p, c.c_char]
+
         if self._load_parser(self._zpar_session_obj, modelpath.encode('utf-8')):
             raise OSError('Cannot find parser model at {}\n'.format(modelpath))
 
@@ -50,8 +58,26 @@ class Parser(object):
         else:
             raise OSError('File {} does not exist.'.format(inputfile))
 
+    def parse_tagged_sentence(self, tagged_sentence, sep='/'):
+        if not tagged_sentence.strip():
+            # return empty string if the input is empty
+            ans = ""
+        else:
+            zpar_compatible_sentence = tagged_sentence.strip().encode('utf-8')
+            parsed_sent = self._parse_tagged_sentence(self._zpar_session_obj, zpar_compatible_sentence, sep.encode('utf-8'))
+            ans = parsed_sent.decode('utf-8')
+        return ans
+
+    def parse_tagged_file(self, inputfile, outputfile, sep='/'):
+        if os.path.exists(inputfile):
+            self._parse_tagged_file(self._zpar_session_obj, inputfile.encode('utf-8'), outputfile.encode('utf-8'), sep.encode('utf-8'))
+        else:
+            raise OSError('File {} does not exist.'.format(inputfile))
+
     def cleanup(self):
         self._load_parser = None
         self._parse_sentence = None
         self._parse_file = None
+        self._parse_tagged_sentence = None
+        self._parse_tagged_file = None
         self._zpar_session_obj = None
